'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var swal = _interopDefault(require('sweetalert2'));

function styleInject(css, ref) {
  if (ref === void 0) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') {
    return;
  }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = "button.microfeedback-button {\n  cursor: pointer;\n  text-decoration: none;\n  position: fixed;\n  bottom: 0;\n  right: 3.125em;\n  padding: 4px 7px;\n  font-size: 0.875em;\n  border-radius: 5px 5px 0 0;\n  z-index: 1001;\n  transition: all 0.2s ease-in-out;\n\n  /* Button reset */\n  border: none;\n  margin: 0;\n  width: auto;\n  overflow: visible;\n  background: transparent;\n  color: inherit;\n\n  /* Normalize `line-height`. Cannot be changed from `normal` in Firefox 4+. */\n  line-height: normal;\n\n  /* Corrects font smoothing for webkit */\n  -webkit-font-smoothing: inherit;\n  -moz-osx-font-smoothing: inherit;\n\n  /* Corrects inability to style clickable `input` types in iOS */\n  -webkit-appearance: none;\n}\n\nbutton.microfeedback-button:hover,\nbutton.microfeedback-button--clicked {\n  /* Darken background color a bit */\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR4nGNikAQAACIAHF/uBd8AAAAASUVORK5CYII=\");\n  padding-bottom: 10px;\n}\n";
styleInject(css);

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var defaults$1 = {
  method: 'POST',
  url: null,
  payload: null
};

var sendJSON = (function (options) {
  var opts = _extends({}, defaults$1, options);
  return new Promise(function (resolve, reject) {
    var req = new XMLHttpRequest();
    req.open(opts.method, opts.url, true);
    req.setRequestHeader('Content-Type', 'application/json');
    req.setRequestHeader('Accept', 'application/json');
    req.send(JSON.stringify(opts.payload));
    req.addEventListener('load', function () {
      if (req.status < 400) {
        var data = JSON.parse(req.response);
        resolve(data);
      } else {
        reject(new Error(req.statusText));
      }
    });
    req.addEventListener('error', function () {
      reject(new Error('Network Error'));
    });
  });
});

// Less typing
var d = document;
var noop = function noop() {};
var clickedClass = 'microfeedback-button--clicked';

var makeButton = function makeButton(options) {
  return '<button aria-label="' + options.buttonAriaLabel + '" style="background-color: ' + options.backgroundColor + '; color: ' + options.color + '" class="microfeedback-button">' + options.buttonText + '</button>';
};
var defaults$2 = {
  url: null,
  buttonText: 'Feedback',
  buttonAriaLabel: 'Send feedback',
  title: 'Send feedback',
  placeholder: 'Describe your issue or share your ideas',
  extra: null,
  backgroundColor: '#3085d6',
  color: '#fff',
  optimistic: true,
  showDialog: function showDialog(btn) {
    var swalOpts = {
      title: btn.options.title,
      input: 'textarea',
      inputPlaceholder: btn.options.placeholder,
      showCancelButton: true,
      confirmButtonText: 'Send'
    };
    if (!btn.options.optimistic) {
      swalOpts.showLoaderOnConfirm = true;
      swalOpts.preConfirm = function (value) {
        return btn.onSubmit({ value: value });
      };
      swalOpts.allowOutsideClick = function () {
        return !swal.isLoading();
      };
    }
    // Allow passing any valid sweetalert2 options
    Object.keys(btn.options).forEach(function (each) {
      if (swal.isValidParameter(each)) {
        swalOpts[each] = btn.options[each];
      }
    });
    return btn.alert(swalOpts);
  },
  showSuccessDialog: function showSuccessDialog(btn) {
    return btn.alert('Thank you!', 'Your feedback has been submitted.', 'success');
  },
  getPayload: function getPayload(btn, _ref) {
    var body = _ref.value;

    var payload = { body: body };
    if (btn.options.extra) {
      payload.extra = btn.options.extra;
    }
    return payload;
  },
  preSend: function preSend(btn, input) {
    if (btn.options.optimistic) {
      // Show thank you message before request is sent so the
      // user doesn't have to wait
      return btn.options.showSuccessDialog(btn, input);
    }
  },
  sendRequest: function sendRequest(btn, url, payload) {
    return sendJSON({
      url: url,
      method: 'POST',
      payload: payload
    });
  },
  onSuccess: function onSuccess(btn, input, response) {
    if (!btn.options.optimistic) {
      return btn.options.showSuccessDialog(btn, input, response);
    }
  },
  onFailure: noop
};

var MicroFeedbackButton = function () {
  function MicroFeedbackButton(element, options) {
    classCallCheck(this, MicroFeedbackButton);

    var opts = element instanceof HTMLElement ? options : element;
    this.options = _extends({}, defaults$2, opts);
    if (!this.options.url) {
      console.warn('options.url not provided. Feedback will only be logged to the console.');
    }

    this.appended = false;
    this._parent = null;
    if (element instanceof HTMLElement) {
      this.el = element;
    } else {
      // assume element is an object
      var buttonParent = d.createElement('div');
      buttonParent.innerHTML = makeButton(this.options);
      d.body.appendChild(buttonParent);
      this._parent = buttonParent;
      this.appended = true;
      this.el = buttonParent.querySelector('.microfeedback-button');
    }
    this.el.addEventListener('click', this.onClick.bind(this), false);
  }

  createClass(MicroFeedbackButton, [{
    key: 'alert',
    value: function alert() {
      return swal.apply(undefined, arguments);
    }
  }, {
    key: 'onSubmit',
    value: function onSubmit(input) {
      // Backend requires body in payload
      if (input.dismiss) {
        return null;
      }
      var payload = this.options.getPayload(this, input);
      // microfeedback backends requires 'body'
      if (payload.body) {
        this.options.preSend(this, input);
        var promise = void 0;
        var url = typeof this.options.url === 'function' ? this.options.url(this, input) : this.options.url;
        if (url) {
          promise = this.options.sendRequest(this, url, payload, input);
        } else {
          console.debug('microfeedback payload:');
          console.debug(payload);
          promise = Promise.resolve(payload);
        }
        return promise.then(this.options.onSuccess.bind(this, this, input), this.options.onFailure.bind(this, this, input));
      }
    }
  }, {
    key: 'onClick',
    value: function onClick(e) {
      var _this = this;

      // eslint-disable-next-line no-unused-expressions
      e && e.preventDefault();
      this.el.classList.add(clickedClass);
      var promise = this.options.showDialog(this).then(function (input) {
        _this.el.classList.remove(clickedClass);
        return input;
      });
      if (this.options.optimistic) {
        promise.then(this.onSubmit.bind(this));
      }
      return promise;
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      this.el.removeEventListener('click', this.onClick.bind(this));
      if (this.appended) {
        d.body.removeChild(this._parent);
      }
    }
  }]);
  return MicroFeedbackButton;
}();

var factory = function factory(element, options) {
  return new MicroFeedbackButton(element, options);
};
factory.MicroFeedbackButton = MicroFeedbackButton;

module.exports = factory;
